/* ====================================================================
 * Copyright (c) 2007,      Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

// sc
#include "RpViewState.h"
#include "Bookmark.h"

// sys
#include <cassert>


static RpViewItemState NullItemState;


RpViewState::RpViewState( Bookmark* bm )
: _pathPos(0)
{
  addPath( bm->getSource() );
}

void RpViewState::setItemState( const sc::String& name, const RpViewItemState& set )
{
  RpViewItemStates::iterator it = _itemStates.find(name);

  if( it != _itemStates.end() )
  {
    RpViewItemState& state = (*it).second;
    state = set;
  }
  else
  {
    _itemStates.insert( RpViewItemStates::value_type(name,set) );
  }
}

const RpViewItemState& RpViewState::getItemState( const sc::String& name )
{
  RpViewItemStates::iterator it = _itemStates.find(name);
  if( it != _itemStates.end() )
  {
    return (*it).second;
  }
  else
  {
    return NullItemState;
  }
}

void RpViewState::getPrevPathHistory( RpPathHistory& history )
{
  for( int i = _pathPos-1; i >= 0; i-- )
  {
    history.push_back( _pathHistory[i] );
  }
}

void RpViewState::getNextPathHistory( RpPathHistory& history )
{
  for( size_t i = _pathPos+1; i < _pathHistory.size(); i++ )
  {
    history.push_back( _pathHistory[i] );
  }
}

void RpViewState::prevPath()
{
  prevPath(1);
}

void RpViewState::nextPath()
{
  nextPath(1);
}

void RpViewState::prevPath( int cnt )
{
  if( _pathPos-cnt >= 0 )
    _pathPos -= cnt;

  if( _pathPos < _pathHistory.size() )
  {
    emit currentChanged( _pathHistory[_pathPos] );
    emit historyChanged();
  }
}

void RpViewState::nextPath( int cnt )
{
  if( _pathPos+cnt < _pathHistory.size() )
    _pathPos += cnt;

  if( _pathPos < _pathHistory.size() )
  {
    emit currentChanged( _pathHistory[_pathPos] );
    emit historyChanged();
  }
}

const sc::String& RpViewState::getPath()
{
  return _pathHistory[_pathPos];
}

void RpViewState::addPath( const sc::String& path )
{
  if( _pathPos+1 < _pathHistory.size() )
  {
    _pathHistory.resize(_pathPos);
  }

  _pathHistory.push_back(path);
  _pathPos = (unsigned int)_pathHistory.size() - 1;

  emit historyChanged();
}
