/* ====================================================================
 * Copyright (c) 2007,      Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

#ifndef _SC_RPVIEWSTATE_H
#define _SC_RPVIEWSTATE_H

// sc
#include "util/String.h"
class Bookmark;

// sys
#include <map>
#include <vector>

// qt
#include <QtCore/QObject>


typedef struct _RpViewItemState {

  _RpViewItemState() : 
    _selected(false), _opened(false)
  {
  }

  bool _selected;
  bool _opened;

} RpViewItemState;


typedef std::map< sc::String, RpViewItemState >  RpViewItemStates;
typedef std::vector< sc::String >                RpPathHistory;


class RpViewState : public QObject
{
  typedef QObject super;
  Q_OBJECT

public:
  RpViewState( Bookmark* );

  void setItemState( const sc::String& name, const RpViewItemState& set );
  const RpViewItemState& getItemState( const sc::String& name );

  void getPrevPathHistory( RpPathHistory& );
  void getNextPathHistory( RpPathHistory& );

  const sc::String& getPath();
  void setPath( const sc::String& path );
  void addPath( const sc::String& path );

public slots:
  void prevPath();
  void nextPath();
  void prevPath( int cnt );
  void nextPath( int cnt );

signals:
  void historyChanged();
  void currentChanged( const sc::String& );
  void optionsChanged();

private:
  RpViewItemStates _itemStates;

  RpPathHistory    _pathHistory;
  unsigned int     _pathPos;
};

#endif // _SC_RPVIEWSTATE_H
