/* ====================================================================
 * Copyright (c) 2007,      Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

#ifndef _SC_RPVIEWTREE_H
#define _SC_RPVIEWTREE_H

// sc
#include "util/String.h"
#include "svn/DirEntryTypes.h"
class RpViewState;
class RpViewModel;
class DirEntryLvi;
class ActionStorage;
class DragInfo;

// qt
#include <Qt3Support/Q3ListView>
class Q3PopupMenu;

// sys
#include <vector>
#include <map>

typedef std::map< sc::String, DirEntryLvi* > ListLvis;
typedef std::vector< DirEntryLvi* >          ChildLvis;


class RpViewTree : public Q3ListView
{
  typedef Q3ListView super;
  Q_OBJECT

public:
  RpViewTree( RpViewModel* model, ActionStorage* as, QWidget* parent );
  ~RpViewTree();

  RpViewState* getState();
  void getSelection( svn::DirEntries& entries );

public slots:
  /** remove old info.. */
  void updateOld( const sc::String& );
  /** ..add new info    */
  void updateNew( const sc::String& );
  /** change the current dir */
  void setCurrentDir( const sc::String& newCurrent );
  /** refresh the view, honoring any option changes */
  void refresh();

  /** update actions after selection change */
  void selectionChanged();

  // Q.. overwrites
  void doubleClicked( Q3ListViewItem* lvi, const QPoint& p, int col );
  void contextMenuRequest( Q3ListViewItem* item, const QPoint& pos, int col );
  void itemRenamed( Q3ListViewItem* item, int col, const QString& text );
  void expanded( Q3ListViewItem* item );
  void collapsed( Q3ListViewItem* item );

  void showEvent( QShowEvent* );
  void dragMoveEvent( QDragMoveEvent* e );
  void dragLeaveEvent( QDragLeaveEvent* e );
  void dropEvent( QDropEvent* e );
  Q3DragObject* dragObject();
  void dragTimeout();

signals:
  /** signal move item, set base if to is basename, else dirname is asumed */
  void move( const sc::String& to, bool base );
  void copy( const sc::String& to );

private:
  RpViewModel*     _model;

  RpViewState*     _state;
  ListLvis         _entries;
  ListLvis         _entriesup;  ///< temporary, filled in updateOld, used/cleaned up in UpdateNew
  sc::String       _current;

  ActionStorage*   _actions;
  Q3PopupMenu*     _menu;

  DragInfo*        _dragInfo;   ///< temporary, drag&drop info
};

#endif // _SC_RPVIEWTREE_H
