/* ====================================================================
 * Copyright (c) 2007,      Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

// sc
#include "config.h"
#include "RpViewTreeWidget.h"
#include "RpViewTree.h"
#include "RpViewState.h"
#include "RpViewModel.h"
#include "sublib/ActionStorage.h"

// qt
#include <QtGui/QLayout>
#include <QtGui/QToolButton>
#include <QtGui/QStatusBar>
#include <QtGui/QLabel>
#include <QtGui/QCheckBox>
#include <Qt3Support/Q3PopupMenu>


RpViewTreeWidget::RpViewTreeWidget( RpViewModel* model, ActionStorage* as, QWidget* parent )
: super(parent)
{
  QVBoxLayout* vl = new QVBoxLayout(this);
  vl->setContentsMargins(0,0,0,2);
  vl->setSpacing(3);
  {
    _tree = new RpViewTree(model, as, this);
    vl->addWidget(_tree);
    vl->setStretchFactor(_tree,1);

    QHBoxLayout* hl = new QHBoxLayout();
    vl->addLayout(hl);
    {
      hl->setSpacing(4);

      _back = new QToolButton(this);
      _back->setArrowType(Qt::LeftArrow);
      _back->setDisabled(true);
      _back->setPopup( new Q3PopupMenu(_back) );
      _back->setPopupDelay(250);
      _back->setFixedWidth(10);
      _back->setAutoRaise(true);

      _forward = new QToolButton(this);
      _back->setArrowType(Qt::RightArrow);
      _forward->setDisabled(true);
      _forward->setPopup( new Q3PopupMenu(_forward) );
      _forward->setPopupDelay(250);
      _forward->setFixedWidth(10);
      _forward->setAutoRaise(true);

      QStatusBar* status = new QStatusBar(this);
      status->setSizeGripEnabled(false);
      status->setContentsMargins(0,0,2,0); // fix drawing error

      _version = new QLabel("head",status);
      status->addWidget( _version, 0, true );
      hl->addSpacing(2);
      hl->addWidget(_back);
      hl->addSpacing(2);
      hl->addWidget(_forward);
      hl->addSpacing(2);
      hl->addWidget(status);
    }
  }

  // back/forward
  connect( _tree->getState(), SIGNAL(historyChanged()), this, SLOT(historyChange()) );
  connect( _tree->getState(), SIGNAL(currentChanged(const sc::String&)),
           _tree,             SLOT(setCurrentDir(const sc::String&)) );
  connect( _back,    SIGNAL(clicked()), _tree->getState(), SLOT(prevPath()) );
  connect( _forward, SIGNAL(clicked()), _tree->getState(), SLOT(nextPath()) );
  connect( _back->popup(),    SIGNAL(activated(int)), _tree->getState(), SLOT(prevPath(int)) );
  connect( _forward->popup(), SIGNAL(activated(int)), _tree->getState(), SLOT(nextPath(int)) );

  // view options
  connect( _tree->getState(), SIGNAL(optionsChanged()), _tree, SLOT(refresh()) );

  connect( _tree, SIGNAL(move(const sc::String&,bool)), model, SLOT(move(const sc::String&,bool)) );
  connect( _tree, SIGNAL(copy(const sc::String&)), model, SLOT(copy(const sc::String&)) );
}

void RpViewTreeWidget::historyChange()
{
  {
    RpPathHistory history;
    _tree->getState()->getPrevPathHistory(history);

    Q3PopupMenu* menu = (Q3PopupMenu*)_back->popup();
    menu->clear();

    int id = 1;
    for( RpPathHistory::iterator it = history.begin(); it != history.end(); it++ )
    {
      menu->insertItem( QString::fromUtf8((*it)), id++ );
    }

    _back->setEnabled( history.size()>0 );
  }

  {
    RpPathHistory history;
    _tree->getState()->getNextPathHistory(history);

    Q3PopupMenu* menu = (Q3PopupMenu*)_forward->popup();
    menu->clear();

    int id = 1;
    for( RpPathHistory::iterator it = history.begin(); it != history.end(); it++ )
    {
      menu->insertItem( QString::fromUtf8((*it)), id++ );
    }

    _forward->setEnabled( history.size()>0 );
  }
}
