/* ====================================================================
 * Copyright (c) 2007,      Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

// sc
#include "config.h"
#include "WcStatusDisplayData.h"
#include "StatusId.h"
#include "svn/WcStatus.h"
#include "svn/WcEntry.h"


WcStatusDisplayData::WcStatusDisplayData( svn::WcStatusPtr status )
: _status(status)
{
}

QString WcStatusDisplayData::data( int column ) const
{
  switch(column)
  {
  case ColName:
    {
      return QString::fromUtf8( _status->getName().getStr() );
    }
  case ColTextStatus:
    {
      return getStatusCode( _status->getTextStatus() );
    }
  case ColPropStatus:
    {
      return getStatusCode( _status->getPropStatus() );
    }
  case ColRpLock:
    {
      return getLockStatusCode( _status->getLockStatus() );
    }
  case ColWcLock:
    {
      return _status->isLocked() ?
        getStatusCode(StatusWcLockLocked) : getStatusCode(StatusWcLockNone);
    }
  case ColCopied:
    {
      return _status->isCopied() ?
        getStatusCode(StatusHistory) : getStatusCode(StatusHistoryNone);
    }
  case ColSwitched:
    {
      return _status->isSwitched() ?
        getStatusCode(StatusSwitched) : getStatusCode(StatusSwitchedNo);
    }
  case ColUpdate:
    {
      bool t = _status->getReposTextStatus() != svn::WcStatus_None;
      bool p = _status->getReposTextStatus() != svn::WcStatus_None;

      return ( t || p ) ?
        getStatusCode(StatusUpToDateNo) : getStatusCode(StatusUpToDate);
    }
  case ColWcRevision:
    {
      const svn::WcEntry* e = _status->getWcEntry();
      if( ! e )
      {
        return "";
      }
      else if( e->getRevnumber() < 0 )
      {
        return "?";
      }
      else if( _status->isCopied() )
      {
        return "-";
      }
      else
      {
        return QString().sprintf( "%ld", (unsigned long)e->getRevnumber() );
      }
    }
  case ColCommitRevision:
    {
      const svn::WcEntry* e = _status->getWcEntry();
      if( ! e )
      {
        return "";
      }
      else if( e->getCmtRevnumber() < 0 )
      {
        return "?";
      }
      else
      {
        return QString().sprintf( "%ld", (unsigned long)e->getCmtRevnumber() );
      }
    }
  case ColCommitAuthor:
    {
      const svn::WcEntry* e = _status->getWcEntry();
      if( ! e )
      {
        return "";
      }
      else if( ! e->getCmtAuthor().getStr() )
      {
        return "?";
      }
      else
      {
        return QString::fromUtf8( e->getCmtAuthor().getStr() );
      }
    }
  case ColDeepStatus:
    {
      // todo
    }
  case ColEmpty:
    return "";
  default:
    return "?";
  }

#if 0
case StatusColumnDeepStatus:
  {
    return super::text(StatusColumnDeepStatus);
  }
#endif
}

Qt::Alignment WcStatusDisplayData::alignment( int column )
{
  switch(column)
  {
  case ColName:
    return Qt::AlignLeft | Qt::AlignVCenter;
  default:
    return Qt::AlignCenter | Qt::AlignVCenter;
  }
}

QColor WcStatusDisplayData::color()
{
  return getStatusColor(_status->getTextStatus());
}

QString WcStatusDisplayData::header( int column )
{
  switch(column)
  {
  case ColName: 
    return _q("path/name");
  case ColTextStatus:
    return _q("ts");
  case ColPropStatus:
    return _q("ps");
  case ColRpLock:
    return _q("rl");
  case ColWcLock:
    return _q("wl");
  case ColCopied:
    return _q("ah");
  case ColSwitched:
    return _q("sw");
  case ColUpdate:
    return _q("upd");
  case ColWcRevision:
    return _q("wc rev");
  case ColCommitRevision:
    return _q("cmt rev");
  case ColCommitAuthor:
    return _q("cmt author");
  case ColDeepStatus:
    return _q("ds");
  case ColEmpty:
    return "";
  default:
    return "?";
  }
}

int WcStatusDisplayData::columns()
{
  return 13;
}
