/* ====================================================================
 * Copyright (c) 2007-2008, Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

#ifndef _SC_WCSTATUSDISPLAYDATA_H
#define _SC_WCSTATUSDISPLAYDATA_H

// sc
#include "svn/WcStatusTypes.h"
#include "util/String.h"

// qt
#include <QtCore/QVariant>



/** make @a svn::WcStatusPtr type available to QVariant. */
Q_DECLARE_METATYPE(svn::WcStatusPtr);

/** make @a sc::String type available to QVariant. */
Q_DECLARE_METATYPE(sc::String);



/** Helper class to extract display data from a @a svn::WcStatusPtr. */
class WcStatusDisplayData
{
public:
  /** The working copy status columns. */
  enum Columns
  {
    ColName,
    ColTextStatus,
    ColPropStatus,
    ColRpLock,
    ColWcLock,
    ColCopied,          ///< add with history
    ColSwitched,
    ColUpdate,
    ColWcRevision,
    ColCommitRevision,
    ColCommitAuthor,
    ColDeepStatus,
    ColEmpty
  };


  /** construct object. */
  WcStatusDisplayData( svn::WcStatusPtr status );

  /** return the data for @a column. */
  QString data( int column ) const;

  /** return the status color. */
  QColor color();

  /** return the header string for @a column. */
  static QString header( int column );

  /** return alignment for @a column. */
  static Qt::Alignment alignment( int column );

  /** return the number of columns. */
  static int columns();

private:
  svn::WcStatusPtr _status;
};



#endif // _SC_WCSTATUSDISPLAYDATA_H
