/* ====================================================================
 * Copyright (c) 2006-2007, Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

#ifndef _SC_WCVIEWFLAT_H
#define _SC_WCVIEWFLAT_H

// sc
#include "TreeView.h"
#include "util/String.h"
#include "svn/WcStatusTypes.h"
class WcViewFlatProxyModel;
class WcViewFlatItemModel;
class WcViewFlatState;
class ActionStorage;
class WcViewModel;
class WcSelection;
class DeepStatus;
class DragInfo;

// qt
class Q3PopupMenu;

// sys
#include <vector>
#include <map>


class WcViewFlat : public TreeView
{
  typedef TreeView super;
  Q_OBJECT

public:
  WcViewFlat( WcViewModel* model, ActionStorage* as, QWidget* parent );
  ~WcViewFlat();

  WcViewFlatState* getState();

public slots:
  /** Remove old info of @a path. */
  void updateOld( const sc::String& path );
  /** Add new info of @a path */
  void updateNew( const sc::String& path );
  /** Handle @a status change of @a path */
  void updateDeepStatus( const sc::String& path, const DeepStatus& status );

  /** Handle update start signal for @a path. */
  void updateStart( const sc::String& path );
  /** Handle update finish signal for @a path. */
  void updateFinish( const sc::String& path );

  /** change the current dir */
  void setCurrentDir( const sc::String& newCurrent );
  /** refresh the view, honoring any option changes */
  void refresh();

  // Q.. overwrites
  void contextMenuEvent( QContextMenuEvent* e );
  /** update actions after selection change */
  void selectionChanged( const QItemSelection& selected, const QItemSelection& deselected );
  
  void mouseDoubleClickEvent( QMouseEvent* e );
  /** Rename the current item to @a text. */
  void renameItem( const QString& text );

  void showEvent( QShowEvent* );
  void mousePressEvent( QMouseEvent* e );
  void mouseMoveEvent( QMouseEvent* e );
  void dragEnterEvent( QDragEnterEvent* e );
  void dragMoveEvent( QDragMoveEvent* e );
  void dragLeaveEvent( QDragLeaveEvent* e );
  void dropEvent( QDropEvent* e );
  void timerEvent( QTimerEvent* e );

signals:
  /** Signal move item, set base if @a to is a basename, else a dirname is asumed. */
  void move( const sc::String& to, bool base );
  void copy( const sc::String& to );
  /** notify current dir change */
  void currentChanged( const sc::String& );

private:
  void updateMenu( const WcSelection& sel );
  void getSelection( svn::WcStatuss& statuss );
  QMimeData* mimeData( const QModelIndexList& list );

  WcViewModel*          _model;
  WcViewFlatItemModel*  _itemModel;
  WcViewFlatProxyModel* _proxyModel;

  WcViewFlatState*      _state;

  ActionStorage*        _actions;
  Q3PopupMenu*          _menu;
};


#endif // _SC_WCVIEWFLAT_H
