/* ====================================================================
 * Copyright (c) 2007,      Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

#ifndef _SC_WCVIEWFLATITEMMODEL_H
#define _SC_WCVIEWFLATITEMMODEL_H

// sc
#include "svn/WcStatusTypes.h"
#include "util/String.h"
class WcViewFlatItem;

// qt
#include <QtCore/QAbstractItemModel>


/**
 * Status detail information.
 */
class WcViewFlatStatus
{
public:
  svn::WcStatusPtr _status;
  bool             _deep;
};

/**
 * Qt4 item model for WcViewFlat.
 */
class WcViewFlatItemModel : public QAbstractItemModel
{
  typedef QAbstractItemModel super;
  Q_OBJECT

public:
  /** Custom Roles. */
  enum Roles {
    WcStatusRole = Qt::UserRole,
    DragRole
  };

  WcViewFlatItemModel();
  ~WcViewFlatItemModel();

  // QAbstractItemModel
  QModelIndex index( int row, int column, const QModelIndex& parent ) const;
  QModelIndex parent( const QModelIndex& index ) const;
  int columnCount( const QModelIndex& parent ) const;
  int rowCount( const QModelIndex& parent ) const;
  QVariant headerData( int section, Qt::Orientation orientation, int role ) const;
  QVariant data( const QModelIndex& index, int role ) const;
  bool setData( const QModelIndex& index, const QVariant& value, int role );
  Qt::ItemFlags flags( const QModelIndex &index ) const;

  /** return the @a WcStatusPtr for @a row. */
  svn::WcStatusPtr getStatus( int row ) const; 
  /** return the detailed status information @a WcViewFlatStatus for @a row. */
  WcViewFlatStatus getStatusX( int row ) const;
  /** find row index of @a name. -1 if not found. */
  QModelIndex index( const sc::String& name ) const;
  
  /** Set the @a deepStatus of @a path. */
  void setDeepStatus( const sc::String& path, bool deepStatus );

public slots:
  void add( const sc::String& path, const svn::WcStatuss&, bool deepStatus );
  /** remove the @a WcStatuss entries from the model. */
  void remove( const sc::String& path, const svn::WcStatuss& );
  
private:
  void remove(const WcViewFlatItem&);
  void print();

  class Member;
  Member* _m;
};


#endif // _SC_WCVIEWFLATITEMMODEL_H
