/* ====================================================================
 * Copyright (c) 2007,      Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

// sc
#include "WcViewFlatProxyModel.h"
#include "WcViewFlatItemModel.h"
#include "svn/WcStatus.h"
#include "util/Compare.h"


WcViewFlatProxyModel::WcViewFlatProxyModel()
: _all(false), _ignored(false), _outofdate(false)
{
  //setDynamicSortFilter(true);
}

WcViewFlatProxyModel::~WcViewFlatProxyModel()
{
}

QVariant WcViewFlatProxyModel::data( const QModelIndex& index, int role ) const
{
  QVariant value = sourceModel()->data( mapToSource(index), role );

  if( index.column() == 0 && role == Qt::DisplayRole || role == Qt::EditRole )
  {
    QString temp = value.toString();
    temp.remove( 0, _current.getCharCnt()+1 );

    if( temp.length() == 0 )
    {
      temp = ".";
      _currentRow = index.row();
    }

    value = temp;
  }

  return value;
}

Qt::ItemFlags WcViewFlatProxyModel::flags( const QModelIndex &index ) const
{
  Qt::ItemFlags flags = sourceModel()->flags( mapToSource(index) );

  if( index.row() == _currentRow )
    flags &= ~(Qt::ItemIsDragEnabled | Qt::ItemIsEditable);

  return flags;
}

bool WcViewFlatProxyModel::filterAcceptsRow( int sourceRow, const QModelIndex& sourceParent ) const
{
  WcViewFlatItemModel* model = dynamic_cast<WcViewFlatItemModel*>(sourceModel());
  WcViewFlatStatus     item  = model->getStatusX(sourceRow); 

  sc::String target = item._status->getName();


  if( target.getCharCnt() < _current.getCharCnt() )
    return false;

  if( target.getCharCnt() == _current.getCharCnt() )
    return target == _current;

  // if it is a child of _current it has a path seperator at _current length + 1!
  char sep = ((const char*)target)[_current.getByteCnt()];
  if( sep != '/' )
    return false;

  if( strncmp(target,_current,_current.getCharCnt()) != 0 )
    return false;

  if( item._status->isChanged() )
    return true;

  if( item._status->isDir() && ! _all )
    return item._deep;

  if( _ignored && item._status->isIgnored() )
    return true;

  if( _outofdate && item._status->isReposChanged() )
    return true;

  if( _all && ! item._status->isIgnored() && ! item._status->isReposChanged() )
    return true;
  
  return false;
}

bool WcViewFlatProxyModel::lessThan(const QModelIndex &left, const QModelIndex &right) const
{
  if( left.column() == 0 )
  {
    WcViewFlatItemModel* model = dynamic_cast<WcViewFlatItemModel*>(sourceModel());
    WcViewFlatStatus leftItem  = model->getStatusX(left.row()); 
    WcViewFlatStatus rightItem = model->getStatusX(right.row()); 

    int result = compare3( 
      leftItem._status->getName(), leftItem._status->isDir(),
      rightItem._status->getName(), rightItem._status->isDir() );

    return result < 0;
  }
  else
  {
    return super::lessThan(left,right);
  }

  return false;
}

void WcViewFlatProxyModel::setCurrentPath( const sc::String& current )
{
  _current = current;
  invalidateFilter();
}

void WcViewFlatProxyModel::setFilterAll( bool on )
{
  _all = on;
}

void WcViewFlatProxyModel::setFilterIgnored( bool on )
{
  _ignored = on;
}

void WcViewFlatProxyModel::setFilterOutOfDate( bool on )
{
  _outofdate = on;
}
