/* ====================================================================
 * Copyright (c) 2007,      Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

#ifndef _SC_WCVIEWFLATPROXYMODEL_H
#define _SC_WCVIEWFLATPROXYMODEL_H

// sc
#include "util/String.h"

// qt
#include <QtGui/QSortFilterProxyModel>

/**
 * Proxy model for WcViewFlat.
 */
class WcViewFlatProxyModel : public QSortFilterProxyModel
{
  typedef QSortFilterProxyModel super;
  Q_OBJECT

public:
  WcViewFlatProxyModel();
  ~WcViewFlatProxyModel();

  QVariant data( const QModelIndex& index, int role ) const;
  Qt::ItemFlags flags( const QModelIndex &index ) const;

  /** Filter @a sourceRow based on the current filter settings. */
  bool filterAcceptsRow( int sourceRow, const QModelIndex& sourceParent ) const;

  /** Compares the content of @a left and @a right index. */
  bool lessThan(const QModelIndex &left, const QModelIndex &right) const;

  /** Set the @a current dir. This is a filter settings. Any path requested 
   *  by calling data() will have it stripped from the path. */
  void setCurrentPath( const sc::String& current );

  /** Show all items. */
  void setFilterAll( bool on );
  /** Show ignored items. */
  void setFilterIgnored( bool on );
  /** Show out of date items. */
  void setFilterOutOfDate( bool on );

private:
  sc::String  _current;      ///< the current dir.
  mutable int _currentRow;   ///< index row of current dir.

  bool _all;                 ///< filter, show all items.
  bool _ignored;             ///< filter, show ignored items.
  bool _outofdate;           ///< filter, show out of date items.
};

#endif // _SC_WCVIEWFLATPROXYMODEL_H
