/* ====================================================================
 * Copyright (c) 2006, 2008  Martin Hauner
 *                           http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

// sc
#include "WcViewFlatState.h"
#include "Bookmark.h"

// sys
#include <cassert>


static WcViewFlatItemState NullItemState;


WcViewFlatState::WcViewFlatState( Bookmark* bm )
: _all(false), _ignored(false), _updates(false), _pathPos(0)
{
  addPath( bm->getSource() );
}

void WcViewFlatState::setItemState( const sc::String& name, const WcViewFlatItemState& set )
{
  WcViewFlatItemStates::iterator it = _itemStates.find(name);

  if( it != _itemStates.end() )
  {
    WcViewFlatItemState& state = (*it).second;
    state = set;
  }
  else
  {
    _itemStates.insert( WcViewFlatItemStates::value_type(name,set) );
  }
}

const WcViewFlatItemState& WcViewFlatState::getItemState( const sc::String& name )
{
  WcViewFlatItemStates::iterator it = _itemStates.find(name);
  if( it != _itemStates.end() )
  {
    return (*it).second;
  }
  else
  {
    return NullItemState;
  }
}

void WcViewFlatState::getPrevPathHistory( PathHistory& history )
{
  for( int i = _pathPos-1; i >= 0; i-- )
  {
    history.push_back( _pathHistory[i] );
  }
}

void WcViewFlatState::getNextPathHistory( PathHistory& history )
{
  for( size_t i = _pathPos+1; i < _pathHistory.size(); i++ )
  {
    history.push_back( _pathHistory[i] );
  }
}

void WcViewFlatState::prevPath()
{
  prevPath(1);
}

void WcViewFlatState::nextPath()
{
  nextPath(1);
}

void WcViewFlatState::prevPath( int cnt )
{
  if( _pathPos-cnt >= 0 )
    _pathPos -= cnt;

  if( _pathPos < _pathHistory.size() )
  {
    emit currentChanged( _pathHistory[_pathPos] );
    emit historyChanged();
  }
}

void WcViewFlatState::nextPath( int cnt )
{
  if( _pathPos+cnt < _pathHistory.size() )
    _pathPos += cnt;

  if( _pathPos < _pathHistory.size() )
  {
    emit currentChanged( _pathHistory[_pathPos] );
    emit historyChanged();
  }
}

const sc::String& WcViewFlatState::getPath()
{
  return _pathHistory[_pathPos];
}

void WcViewFlatState::addPath( const sc::String& path )
{
  if( _pathPos+1 < _pathHistory.size() )
  {
    _pathHistory.resize(_pathPos+1);
  }

  _pathHistory.push_back(path);
  _pathPos = (unsigned int)_pathHistory.size() - 1;

  emit historyChanged();
}

bool WcViewFlatState::getViewAll()
{
  return _all;
}

bool WcViewFlatState::getViewIgnored()
{
  return _ignored;
}

bool WcViewFlatState::getViewUpdates()
{
  return _updates;
}

void WcViewFlatState::setViewAll( bool on )
{
  _all = on;
  
  emit optionsChanged();
}

void WcViewFlatState::setViewIgnored( bool on )
{
  _ignored = on;

  emit optionsChanged();
}

void WcViewFlatState::setViewUpdates( bool on )
{
  _updates = on;
}
