/* ====================================================================
 * Copyright (c) 2006-2007, Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

#ifndef _SC_WCVIEWFLATSTATE_H
#define _SC_WCVIEWFLATSTATE_H

// sc
#include "util/String.h"
class Bookmark;

// sys
#include <map>
#include <vector>

// qt
#include <QtCore/QObject>


typedef struct _WcViewFlatItemState {

  _WcViewFlatItemState() : 
    _selected(false), _opened(false)
  {
  }

  bool _selected;
  bool _opened;

} WcViewFlatItemState;


typedef std::map< sc::String, WcViewFlatItemState >  WcViewFlatItemStates;
typedef std::vector< sc::String >                    PathHistory;


class WcViewFlatState : public QObject
{
  typedef QObject super;
  Q_OBJECT

public:
  WcViewFlatState( Bookmark* );

  void setItemState( const sc::String& name, const WcViewFlatItemState& set );
  const WcViewFlatItemState& getItemState( const sc::String& name );

  void getPrevPathHistory( PathHistory& );
  void getNextPathHistory( PathHistory& );

  const sc::String& getPath();
  void setPath( const sc::String& path );
  void addPath( const sc::String& path );

  bool getViewAll();
  bool getViewIgnored();
  bool getViewUpdates();

public slots:
  void prevPath();
  void nextPath();
  void prevPath( int cnt );
  void nextPath( int cnt );
  void setViewAll( bool on );
  void setViewIgnored( bool on );
  void setViewUpdates( bool on );

signals:
  void historyChanged();
  void currentChanged( const sc::String& );
  void optionsChanged();

//public slots:

private:
  bool                 _all;
  bool                 _ignored;
  bool                 _updates;

  WcViewFlatItemStates _itemStates;

  PathHistory          _pathHistory;
  unsigned int         _pathPos;
};


#endif // _SC_WCVIEWFLATSTATE_H
