/* ====================================================================
 * Copyright (c) 2007-2008  Martin Hauner
 *                          http://subcommander.tigris.org
 *
 * Subcommander is licensed as described in the file doc/COPYING, which
 * you should have received as part of this distribution.
 * ====================================================================
 */

// sc
#include "WcViewTreeProxyModel.h"
#include "WcViewTreeItemModel.h"
#include "WcStatusDisplayData.h"
#include "svn/WcStatus.h"
#include "svn/Path.h"
#include "util/Compare.h"


WcViewTreeProxyModel::WcViewTreeProxyModel()
: _all(false), _ignored(false), _outofdate(false), _modifiedFolder(false)
{
}

WcViewTreeProxyModel::~WcViewTreeProxyModel()
{
}

QVariant WcViewTreeProxyModel::data( const QModelIndex& index, int role ) const
{
  QVariant value = sourceModel()->data( mapToSource(index), role );

  if( index.column() == 0 && role == Qt::DisplayRole || role == Qt::EditRole )
  {
    QString temp = value.toString();

    if( temp == (const char*)_current )
    {
      temp = ".";
      _currentRow = index.row();
    }
    else
    {
      temp = temp.section('/', -1);
    }

    value = temp;
  }

  return value;
}

Qt::ItemFlags WcViewTreeProxyModel::flags( const QModelIndex &index ) const
{
  Qt::ItemFlags flags = sourceModel()->flags( mapToSource(index) );

  if( index.row() == _currentRow )
    flags &= ~(Qt::ItemIsDragEnabled | Qt::ItemIsEditable);

  return flags;
}

static sc::String breakpoint("/Users/hauner/Development/Subcommander/sc/po");
bool WcViewTreeProxyModel::filterAcceptsRow( int sourceRow, const QModelIndex& sourceParent ) const
{
  WcViewTreeItemModel* model = dynamic_cast<WcViewTreeItemModel*>(sourceModel());

  QModelIndex index = model->index( sourceRow, 0, sourceParent );

  svn::WcStatusPtr status = index.data(WcViewTreeItemModel::WcStatusRole).value<svn::WcStatusPtr>();
  sc::String target = status->getName();

  if( target == _current )
    return true;

  // filter siblings of current root index
  sc::String targetDir  = svn::Path::getDirName(target);
  sc::String currentDir = svn::Path::getDirName(_current);
  if( targetDir == currentDir )
    return false;

  if( status->isChanged() )
    return true;

  if( status->isDir() )
  {
    if( _modifiedFolder )
    {
      bool deep = index.data(WcViewTreeItemModel::DeepRole).toBool();
      
      if(deep && target == breakpoint)
        int i = 0;
      
      return deep;
    }

    return true;    
  }

  if( _ignored && status->isIgnored() )
    return true;

  if( _outofdate && status->isReposChanged() )
    return true;

  if( _all && ! status->isIgnored() && ! status->isReposChanged() )
    return true;
  
  return false;
}

bool WcViewTreeProxyModel::lessThan(const QModelIndex &left, const QModelIndex &right) const
{
  if( left.column() == 0 && left.isValid() && right.isValid() )
  {
    WcViewTreeItemModel* model = dynamic_cast<WcViewTreeItemModel*>(sourceModel());

    svn::WcStatusPtr leftItem = model->data( left, WcViewTreeItemModel::WcStatusRole ).
      value<svn::WcStatusPtr>();
    svn::WcStatusPtr rightItem = model->data( right, WcViewTreeItemModel::WcStatusRole ).
      value<svn::WcStatusPtr>();

    int result = compare3( 
      leftItem->getName(), leftItem->isDir(),
      rightItem->getName(), rightItem->isDir() );

    return result < 0;
  }
  else
  {
    return super::lessThan(left,right);
  }

  return false;
}

void WcViewTreeProxyModel::setCurrentPath( const sc::String& current )
{
  _current = current;
  invalidateFilter();
}

void WcViewTreeProxyModel::setFilterAll( bool on )
{
  _all = on;
}

void WcViewTreeProxyModel::setFilterIgnored( bool on )
{
  _ignored = on;
}

void WcViewTreeProxyModel::setFilterOutOfDate( bool on )
{
  _outofdate = on;
}

void WcViewTreeProxyModel::setFilterModifiedFolder( bool on )
{
  _modifiedFolder = on;
}
